'use strict';

var preact = require('preact');
var rxjs = require('rxjs');
var operators = require('rxjs/operators');

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

var hasOwn = Object.prototype.hasOwnProperty; // Object.is polyfill

var is = function is(x, y) {
  if (x === y) {
    return x !== 0 || y !== 0 || 1 / x === 1 / y;
  } else {
    // eslint-disable-next-line no-self-compare
    return x !== x && y !== y;
  }
};

var shallowEqual = function shallowEqual(a, b) {
  if (is(a, b)) {
    return true;
  }

  if (typeof a !== 'object' || a === null || typeof b !== 'object' || b === null) {
    return false;
  } // eslint-disable-next-line no-self-compare


  if (Array.isArray(a) !== Array.isArray(b)) {
    return false;
  }

  var keysA = Object.keys(a);
  var keysB = Object.keys(b);

  if (keysA.length !== keysB.length) {
    return false;
  }

  for (var i = 0; i < keysA.length; i++) {
    if (!hasOwn.call(b, keysA[i]) || !is(a[keysA[i]], b[keysA[i]])) {
      return false;
    }
  }

  return true;
};

var BassdrumComponent =
/*#__PURE__*/
function (_Component) {
  _inheritsLoose(BassdrumComponent, _Component);

  function BassdrumComponent(props, context) {
    var _this;

    _this = _Component.call(this, props, context) || this;
    _this.propsStream = new rxjs.BehaviorSubject(props);
    _this.updates = new rxjs.Subject();
    _this.subscriptions = [];

    _this.subscriptions.push(_this.createSubscription());

    return _this;
  }

  var _proto = BassdrumComponent.prototype;

  _proto.createSubscription = function createSubscription() {
    return new rxjs.Subscription();
  };

  _proto.componentDidMount = function componentDidMount() {
    this.updates.next(this.props);
  };

  _proto.componentWillReceiveProps = function componentWillReceiveProps(props) {
    this.propsStream.next(props);
  };

  _proto.shouldComponentUpdate = function shouldComponentUpdate(_, nextState) {
    return this.state !== nextState;
  };

  _proto.componentDidUpdate = function componentDidUpdate() {
    this.updates.next(this.props);
  };

  _proto.render = function render() {
    return preact.h(preact.Fragment, null);
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.propsStream.complete();
    this.updates.complete();
    this.subscriptions.forEach(function (sub) {
      return sub.unsubscribe();
    });
    this.subscriptions.length = 0;
  };

  return BassdrumComponent;
}(preact.Component);

var createComponentAPI = function createComponentAPI(component) {
  return {
    props: component.propsStream,
    subscribe: function subscribe(obs) {
      return component.subscriptions.push(obs.subscribe());
    },
    updates: component.updates
  };
};

var createComponent = function createComponent(componentFunction, template) {
  return (
    /*#__PURE__*/
    function (_BassdrumComponent) {
      _inheritsLoose(_class, _BassdrumComponent);

      function _class() {
        return _BassdrumComponent.apply(this, arguments) || this;
      }

      var _proto2 = _class.prototype;

      _proto2.createSubscription = function createSubscription() {
        var _this2 = this;

        var hasEmitted = false;
        var subscription = componentFunction(createComponentAPI(this)).pipe(operators.distinctUntilChanged(shallowEqual)).subscribe(function (state) {
          hasEmitted = true;

          _this2.setState(state);
        });

        {
          if (!hasEmitted) {
            throw new Error('Your Component did not emit any state when it was created. ' + 'Make sure the Observable you return from your ' + 'component function emits immediately.');
          }
        }

        return subscription;
      };

      _proto2.render = function render() {
        return template(this.state);
      };

      return _class;
    }(BassdrumComponent)
  );
};

function combine() {
  var sources = [];

  for (var i = 0, l = arguments.length; i < l; i++) {
    var stream = i < 0 || arguments.length <= i ? undefined : arguments[i];

    if (rxjs.isObservable(stream)) {
      sources.push(stream);
    } else {
      var _loop = function _loop(key) {
        var _of;

        var value = stream[key];
        sources.push(rxjs.isObservable(value) ? value.pipe(operators.map(function (value) {
          var _ref;

          return _ref = {}, _ref[key] = value, _ref;
        })) : rxjs.of((_of = {}, _of[key] = value, _of)));
      };

      for (var key in stream) {
        _loop(key);
      }
    }
  }

  return rxjs.combineLatest.apply(void 0, sources).pipe(operators.map(function (values) {
    return Object.assign.apply(Object, [{}].concat(values));
  }));
}

var createHandler = function createHandler() {
  var subject = new rxjs.Subject();

  var handleEvent = function handleEvent(event) {
    return subject.next(event);
  };

  return [handleEvent, subject];
};

var createRef = function createRef() {
  var subject = new rxjs.BehaviorSubject(null);

  var ref = function ref(el) {
    subject.next(el);
  };

  return [ref, subject];
};

exports.combine = combine;
exports.createComponent = createComponent;
exports.createHandler = createHandler;
exports.createRef = createRef;
//# sourceMappingURL=bassdrum.cjs.development.js.map
